use Test::More tests => 1;
use_ok( 'POE::Component::Metabase::Client::Submit' );
diag( "Testing POE::Component::Metabase::Client::Submit-$POE::Component::Metabase::Client::Submit::VERSION, POE-$POE::VERSION, Perl $], $^X" );
